--[[
Name: LibAuraInfo-1.0
Author(s): Cyprias (cyprias@gmail.com)
Documentation: http://www.wowace.com/addons/libaurainfo-1-0/
SVN: svn://svn.wowace.com/wow/libaurainfo-1-0/mainline/trunk
Description: Database of spellID's duration and debuff type.
Dependencies: LibStub
]]

local _G = _G
local print = print
local UnitAura = UnitAura
local _
local math_floor = math.floor
local UnitIsUnit = UnitIsUnit
local UnitExists = UnitExists
local UnitName = UnitName
local UnitGUID = UnitGUID
local pairs = pairs
local table_insert = table.insert
local table_remove = table.remove
local GetSpellInfo = GetSpellInfo
local UnitIsPlayer = UnitIsPlayer
local GetTime = GetTime


local MAJOR, MINOR = "LibAuraInfo-1.0", 1
if not LibStub then error(MAJOR .. " requires LibStub.") return end

local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

lib.callbacks = lib.callbacks or LibStub("CallbackHandler-1.0"):New(lib)
if not lib.callbacks then	error(MAJOR .. " CallbackHandler-1.0.") return end




lib.GUIDDurations = {}

local DEBUG = false
--[===[@debug@
DEBUG = true
--@end-debug@]===]

local function debugPrint(...)
	if DEBUG then
		print(...)
	end
end
lib.debugPrint = debugPrint

--Save debuffType as a number, then return as a string when requested.
lib.debuffTypes = {
	Magic = 1,
	Disease = 2,
	Poison = 3,
	Curse = 4,
	[1] = "Magic",
	[2] = "Disease",
	[3] = "Poison",
	[4] = "Curse",
}

lib.trackAuras = false
lib.callbacksUsed = {}

--~ lib.callbacks:Fire("LibAuraInfo-1.0_xxxx", "blaw")
function lib.callbacks:OnUsed(target, eventname)
--~ 	debugPrint("OnUsed", target, eventname)
	
	lib.callbacksUsed[eventname] = lib.callbacksUsed[eventname] or {}
	table_insert(lib.callbacksUsed[eventname], #lib.callbacksUsed[eventname]+1, target)
	lib.trackAuras = true
	lib.frame:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
end

function lib.callbacks:OnUnused(target, eventname)
--~ 	debugPrint("OnUsed", target, eventname)
	if lib.callbacksUsed[eventname] then
		for i= #lib.callbacksUsed[eventname], 1, -1 do 
			if lib.callbacksUsed[eventname][i] == target then
				table_remove(lib.callbacksUsed[eventname], i)
			end
		end
	end
	
	for event, value in pairs(lib.callbacksUsed) do 
		if #value == 0 then
			lib.callbacksUsed[event] = nil
		end
	end
	
	for event in pairs(lib.callbacksUsed) do 
		return
	end
	lib.trackAuras = false
	lib.frame:UnregisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
end


local LAI_DB
local function Round(num, zeros)
	return math_floor( num * 10 ^ (zeros or 0) + 0.5 ) / 10 ^ (zeros or 0)
end

local function ResetUnitAuras(unitID)
	if lib.trackAuras  == true then
		local dstGUID = UnitGUID(unitID)
		lib:RemoveAllAurasFromGUID(dstGUID)
	end
end

function lib:AddAuraFromUnitID(dstGUID, spellID, srcGUID, isDebuff, debuffType, duration, expirationTime, count)
	local key = dstGUID.."-"..spellID
	
	self.GUIDAuras[dstGUID] = self.GUIDAuras[dstGUID] or {}
	table_insert(self.GUIDAuras[dstGUID], #self.GUIDAuras[dstGUID]+1, spellID)
	
	self.GUIDAuraData_casterGUID[key] = srcGUID
	self.GUIDAuraData_duration[key] = duration
	self.GUIDAuraData_expirationTime[key] = expirationTime
	self.GUIDAuraData_isDebuff[key] = isDebuff
	self.GUIDAuraData_debuffType[key] = debuffType
	if count and count > 1 then
		self.GUIDAuraData_stackCount[key] = count
	end
end

local function CheckUnitAuras(unitID, filterType)
--~ 	name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId  = UnitAura("unit", index or "name"[, "rank"[, "filter"]]) 
	local i = 1;
	local name, rank, dispelType, duration, isStealable, spellID, unitCaster, expirationTime, count, shouldConsolidate
	
	local dstGUID = UnitGUID(unitID)
	local srcGUID
	
	while true do 
		name, rank, _, count, dispelType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellID = UnitAura(unitID, i, filterType)
		if not name then break end

		duration = Round(duration)

		if not lib.auraInfo[spellID] then

			lib.auraInfo[spellID] = (duration or 0) .. ";" .. (dispelType and lib.debuffTypes[dispelType] or 0)--add it temporarily.
			
			--[===[@debug@
			if not LAI_DB.new[spellID] then
				--Save the spell info to a saved variable to add it later. End users won't execute this code.
				LAI_DB.new[spellID] = lib.auraInfo[spellID]    .."  --"..name .. (rank and rank ~= "" and " ("..rank..")" or "") .."*"
				debugPrint("Missing info on", UnitName(unitID), name, rank, spellID, dispelType, duration)
			end
			--@end-debug@]===]
		elseif not lib.auraInfoPvP[spellID] then
			if unitCaster and UnitExists(unitCaster) then
				local GUID = UnitGUID(unitCaster)
				local baseDuration = lib:GetDuration(spellID) --, nil, nil, UnitIsPlayer(unitID)
				if baseDuration and baseDuration ~= duration then
--~ 					local spellName = GetSpellInfo(spellID)
					
					if duration > 0 then-- Sometimes UnitAura says a spell has 0 duration when it realy has more.
						--caster's duration doesn't match our DB, they're probably speced into something. lets remember that.
						lib.GUIDDurations[GUID.."-"..spellID] = duration
					end
					
					--[===[@debug@
					--If we haven't confirmed the duration on WoWHead and the duration doesn't match our DB duration, save to SV file to confirm duration later.
					if not lib.confirmedDur[spellID] and not lib.auraInfoPvP[spellID] then
						--Save the spell info to a saved variable to add it later. End users won't execute this code.
						if not LAI_DB.needConfirm[spellID] then
							if baseDuration == 0 then
								debugPrint("Replace duration on", spellID, dispelType, name, rank, "ourDur:"..baseDuration, "theirDur:"..duration)
							else
								debugPrint("Confirm duration on", spellID, dispelType, name, rank, "ourDur:"..baseDuration, "theirDur:"..duration)
							end
						end
						
						if baseDuration == 0 then
							LAI_DB.replace[spellID] = (duration or 0) .. ";" .. (dispelType and lib.debuffTypes[dispelType] or 0)    .."  --"..name .. (rank and rank ~= "" and " ("..rank..")" or "") .."* Replace Duration"
						else
							LAI_DB.needConfirm[spellID] = (duration or 0) .. ";" .. (dispelType and lib.debuffTypes[dispelType] or 0)    .."  --"..name .. (rank and rank ~= "" and " ("..rank..")" or "") .."* Confirm Duration"
						end
					end
					--@end-debug@]===]
					
				end
			end
		end
		
		if lib.trackAuras == true and unitCaster then
			srcGUID = UnitGUID(unitCaster)
--~ 			lib:AddAuraToGUID(dstGUID, spellID, srcGUID, filterType == "HARMFUL")
			lib:AddAuraFromUnitID(dstGUID, spellID, srcGUID, filterType == "HARMFUL", dispelType, duration, expirationTime, count)
		end
		
		i = i + 1
	end
end

lib.frame = lib.frame or CreateFrame("Frame")
local function OnEvent(self, event, ...)
	if self[event] then
		self[event](self, event, ...)
	end
end

function lib.frame:UPDATE_MOUSEOVER_UNIT()
	ResetUnitAuras("mouseover")
	CheckUnitAuras("mouseover", "HELPFUL")
	CheckUnitAuras("mouseover", "HARMFUL")
end

function lib.frame:PLAYER_TARGET_CHANGED()
	ResetUnitAuras("target")
	CheckUnitAuras("target", "HELPFUL")
	CheckUnitAuras("target", "HARMFUL")
end

function lib.frame:UNIT_TARGET(unitID)
	if not UnitIsUnit(unitID, "player") then
		local targetID = unitID.."target"
		ResetUnitAuras(targetID)
		CheckUnitAuras(targetID, "HELPFUL")
		CheckUnitAuras(targetID, "HARMFUL")
	end
end

function lib.frame:UNIT_AURA(unitID)
	ResetUnitAuras(unitID)
	CheckUnitAuras(unitID, "HELPFUL")
	CheckUnitAuras(unitID, "HARMFUL")
end

function lib.frame:PLAYER_LOGOUT(unitID)
	--[===[@debug@
	_G.LAI_DB = LAI_DB
	--@end-debug@]===]
end

function lib.frame:ADDON_LOADED()
	--[===[@debug@
	LAI_DB = _G.LAI_DB or {needConfirm = {}, new = {}}
	self:RegisterEvent('PLAYER_LOGOUT')
	--@end-debug@]===]
	self:RegisterEvent('UPDATE_MOUSEOVER_UNIT')
	self:RegisterEvent('PLAYER_TARGET_CHANGED')
	self:RegisterEvent('UNIT_TARGET')
	self:RegisterEvent('UNIT_AURA')
end

function lib.frame:PLAYER_LOGIN()
	self:ADDON_LOADED()
end

lib.frame:SetScript("OnEvent", OnEvent)
lib.frame:RegisterEvent('ADDON_LOADED')
lib.frame:RegisterEvent('PLAYER_LOGIN')
